#include "METOOLS/Explicit/Lorentz_Calculator.H"
#include "METOOLS/Currents/C_Scalar.H"
#include "METOOLS/Currents/C_Spinor.H"
#include "METOOLS/Currents/C_Vector.H"
#include "METOOLS/Explicit/Vertex.H"
#include "MODEL/Main/Single_Vertex.H"
#include "ATOOLS/Org/Message.H"
#include "ATOOLS/Org/Exception.H"

typedef std::complex<double> complex;

namespace METOOLS {

  template <typename SType>
  class FFV10_Calculator: public Lorentz_Calculator {
  public:
    
    FFV10_Calculator(const Vertex_Key &key):
      Lorentz_Calculator(key) {}

    std::string Label() const { return "FFV10"; }

    CObject *Evaluate(const CObject_Vector &jj)
    {

// if outgoind UFO-index is 0
if (p_v->V()->id.back()==0){
const CSpinor <SType> & j1 = ((jj[0]->Get< CSpinor <SType> >())->B() == -1) ? (*(jj[0]->Get< CSpinor <SType> >())) : (*(jj[0]->Get< CSpinor <SType> >())).CConj() ;
const CVec4 <SType> & j2 = *(jj[1]->Get< CVec4 <SType> >());
CSpinor<SType>* j0 = NULL;
j0 = CSpinor<SType>::New(m_r[0],-1,0,0,0,0,3);
(*j0)[0] = -((j1[2]) * (j2[0]) + (j1[3]) * (j2[ATOOLS::Spinor<SType>::R1()])) + (complex(-0.0,-1.0)) * ((j1[3]) * (j2[ATOOLS::Spinor<SType>::R2()])) - ((j1[2]) * (j2[ATOOLS::Spinor<SType>::R3()]));
(*j0)[1] = -((j1[3]) * (j2[0]) + (j1[2]) * (j2[ATOOLS::Spinor<SType>::R1()])) + (complex(0.0,1.0)) * ((j1[2]) * (j2[ATOOLS::Spinor<SType>::R2()])) + (j1[3]) * (j2[ATOOLS::Spinor<SType>::R3()]);
(*j0)[2] = (j1[0]) * (j2[0]) - ((j1[1]) * (j2[ATOOLS::Spinor<SType>::R1()])) + (complex(-0.0,-1.0)) * ((j1[1]) * (j2[ATOOLS::Spinor<SType>::R2()])) - ((j1[0]) * (j2[ATOOLS::Spinor<SType>::R3()]));
(*j0)[3] = (j1[1]) * (j2[0]) - ((j1[0]) * (j2[ATOOLS::Spinor<SType>::R1()])) + (complex(0.0,1.0)) * ((j1[0]) * (j2[ATOOLS::Spinor<SType>::R2()])) + (j1[1]) * (j2[ATOOLS::Spinor<SType>::R3()]);
j0->SetS(j1.S()|j2.S());
return j0;

}

// if outgoind UFO-index is 1
if (p_v->V()->id.back()==1){
const CSpinor <SType> & j0 = ((jj[1]->Get< CSpinor <SType> >())->B() == 1) ? (*(jj[1]->Get< CSpinor <SType> >())) : (*(jj[1]->Get< CSpinor <SType> >())).CConj() ;
const CVec4 <SType> & j2 = *(jj[0]->Get< CVec4 <SType> >());
CSpinor<SType>* j1 = NULL;
j1 = CSpinor<SType>::New(m_r[1],1,0,0,0,0,3);
(*j1)[0] = (j0[2]) * (j2[0]) - ((j0[3]) * (j2[ATOOLS::Spinor<SType>::R1()])) + (complex(0.0,1.0)) * ((j0[3]) * (j2[ATOOLS::Spinor<SType>::R2()])) - ((j0[2]) * (j2[ATOOLS::Spinor<SType>::R3()]));
(*j1)[1] = (j0[3]) * (j2[0]) - ((j0[2]) * (j2[ATOOLS::Spinor<SType>::R1()])) + (complex(-0.0,-1.0)) * ((j0[2]) * (j2[ATOOLS::Spinor<SType>::R2()])) + (j0[3]) * (j2[ATOOLS::Spinor<SType>::R3()]);
(*j1)[2] = -((j0[0]) * (j2[0]) + (j0[1]) * (j2[ATOOLS::Spinor<SType>::R1()])) + (complex(0.0,1.0)) * ((j0[1]) * (j2[ATOOLS::Spinor<SType>::R2()])) - ((j0[0]) * (j2[ATOOLS::Spinor<SType>::R3()]));
(*j1)[3] = -((j0[1]) * (j2[0]) + (j0[0]) * (j2[ATOOLS::Spinor<SType>::R1()])) + (complex(-0.0,-1.0)) * ((j0[0]) * (j2[ATOOLS::Spinor<SType>::R2()])) + (j0[1]) * (j2[ATOOLS::Spinor<SType>::R3()]);
j1->SetS(j0.S()|j2.S());
return j1;

}

// if outgoind UFO-index is 2
if (p_v->V()->id.back()==2){
const CSpinor <SType> & j0 = ((jj[0]->Get< CSpinor <SType> >())->B() == 1) ? (*(jj[0]->Get< CSpinor <SType> >())) : (*(jj[0]->Get< CSpinor <SType> >())).CConj() ;
const CSpinor <SType> & j1 = ((jj[1]->Get< CSpinor <SType> >())->B() == -1) ? (*(jj[1]->Get< CSpinor <SType> >())) : (*(jj[1]->Get< CSpinor <SType> >())).CConj() ;
CVec4<SType>* j2 = NULL;
j2 = CVec4<SType>::New();
(*j2)[0] = (j0[2]) * (j1[0]) + (j0[3]) * (j1[1]) - ((j0[0]) * (j1[2])) - ((j0[1]) * (j1[3]));
(*j2)[ATOOLS::Spinor<SType>::R1()] = (j0[3]) * (j1[0]) + (j0[2]) * (j1[1]) + (j0[1]) * (j1[2]) + (j0[0]) * (j1[3]);
(*j2)[ATOOLS::Spinor<SType>::R2()] = (complex(0.0,-1.0)) * ((j0[3]) * (j1[0]) - ((j0[2]) * (j1[1])) + (j0[1]) * (j1[2]) - ((j0[0]) * (j1[3])));
(*j2)[ATOOLS::Spinor<SType>::R3()] = (j0[2]) * (j1[0]) - ((j0[3]) * (j1[1])) + (j0[0]) * (j1[2]) - ((j0[1]) * (j1[3]));
j2->SetS(j0.S()|j1.S());
return j2;

}

    }

  };// end of class FFV10_Calculator

  template class FFV10_Calculator<double>;

}// end of namespace METOOLS

using namespace METOOLS;

DECLARE_GETTER(FFV10_Calculator<double>,"DFFV10",
	       Lorentz_Calculator,Vertex_Key);
Lorentz_Calculator *ATOOLS::Getter
<Lorentz_Calculator,Vertex_Key,FFV10_Calculator<double> >::
operator()(const Vertex_Key &key) const
{ return new FFV10_Calculator<double>(key); }

void ATOOLS::Getter<Lorentz_Calculator,Vertex_Key,
		    FFV10_Calculator<double> >::
PrintInfo(std::ostream &str,const size_t width) const
{ str<<"FFV10 vertex"; }
