#include "ATOOLS/Math/MyComplex.H"
#include "ATOOLS/Org/Message.H"
#include "ATOOLS/Phys/Flavour.H"
#include "ATOOLS/Org/Data_Reader.H"
#include "MODEL/UFO/UFO_Model.H"
#include "MODEL/Main/Single_Vertex.H"
#include "MODEL/UFO/UFO_Color_Functions.H"
#include <iomanip>

namespace MODEL{

  class DMsimp_Y1_XD : public UFO::UFO_Model
  {
  public:
    DMsimp_Y1_XD(std::string path, std::string file, bool elementary) : UFO_Model(path, file, elementary) 
    {
      m_name = std::string("DMsimp_Y1_XD");
      ParticleInit();
      ParamInit();
      SetMassiveFlags();
      SetStableFlags();
      SetSMMasses();
      // Massive and Stable flags
      // are set consistently with
      // UFO above, ReadParticleData
      // allows to overwrite these flags.
      // Need this before AddStandardContainers.
      ReadParticleData();
      AddStandardContainers();
      CustomContainerInit();
      ATOOLS::OutputParticles(ATOOLS::msg->Info());
      ATOOLS::OutputContainers(ATOOLS::msg->Info());
    }
  protected:
    void ParticleInit()
    {
      ATOOLS::s_kftable[kf_none] = new ATOOLS::Particle_Info(kf_none,-1,0,0,0,0,-1,0,1,0,"no_particle","no_particle","no_particle","no_particle",1,1);
      
    ATOOLS::s_kftable[22] = new ATOOLS::Particle_Info(22, 1000.0, 0.0, 0, 0, 2, -1, 1, 0, 0, "a", "a", "a", "a");
    ATOOLS::s_kftable[23] = new ATOOLS::Particle_Info(23, 1000.0, 0.0, 0, 0, 2, -1, 1, 0, 1, "Z", "Z", "Z", "Z");
    ATOOLS::s_kftable[24] = new ATOOLS::Particle_Info(24, 1000.0, 0.0, 3, 0, 2, 0, 1, 0, 1, "W+", "W-", "W+", "W-");
    ATOOLS::s_kftable[21] = new ATOOLS::Particle_Info(21, 1000.0, 0.0, 0, 8, 2, -1, 1, 0, 0, "g", "g", "g", "g");
    ATOOLS::s_kftable[9000001] = new ATOOLS::Particle_Info(9000001, 1000.0, 0.0, 0, 0, -2, 0, 1, 0, 0, "ghA", "ghA~", "ghA", "ghA~");
    ATOOLS::s_kftable[9000002] = new ATOOLS::Particle_Info(9000002, 1000.0, 0.0, 0, 0, -2, 0, 1, 0, 1, "ghZ", "ghZ~", "ghZ", "ghZ~");
    ATOOLS::s_kftable[9000003] = new ATOOLS::Particle_Info(9000003, 1000.0, 0.0, 3, 0, -2, 0, 1, 0, 1, "ghWp", "ghWp~", "ghWp", "ghWp~");
    ATOOLS::s_kftable[9000004] = new ATOOLS::Particle_Info(9000004, 1000.0, 0.0, -3, 0, -2, 0, 1, 0, 1, "ghWm", "ghWm~", "ghWm", "ghWm~");
    ATOOLS::s_kftable[82] = new ATOOLS::Particle_Info(82, 1000.0, 0.0, 0, 8, -2, 0, 1, 0, 0, "ghG", "ghG~", "ghG", "ghG~");
    ATOOLS::s_kftable[12] = new ATOOLS::Particle_Info(12, 1000.0, 0.0, 0, 0, 1, 0, 1, 0, 0, "ve", "ve~", "ve", "ve~");
    ATOOLS::s_kftable[14] = new ATOOLS::Particle_Info(14, 1000.0, 0.0, 0, 0, 1, 0, 1, 0, 0, "vm", "vm~", "vm", "vm~");
    ATOOLS::s_kftable[16] = new ATOOLS::Particle_Info(16, 1000.0, 0.0, 0, 0, 1, 0, 1, 0, 0, "vt", "vt~", "vt", "vt~");
    ATOOLS::s_kftable[11] = new ATOOLS::Particle_Info(11, 1000.0, 0.0, -3, 0, 1, 0, 1, 0, 0, "e-", "e+", "e-", "e+");
    ATOOLS::s_kftable[13] = new ATOOLS::Particle_Info(13, 1000.0, 0.0, -3, 0, 1, 0, 1, 0, 0, "mu-", "mu+", "mu-", "mu+");
    ATOOLS::s_kftable[15] = new ATOOLS::Particle_Info(15, 1000.0, 0.0, -3, 0, 1, 0, 1, 0, 1, "ta-", "ta+", "ta-", "ta+");
    ATOOLS::s_kftable[2] = new ATOOLS::Particle_Info(2, 1000.0, 0.0, 2, 3, 1, 0, 1, 0, 0, "u", "u~", "u", "u~");
    ATOOLS::s_kftable[4] = new ATOOLS::Particle_Info(4, 1000.0, 0.0, 2, 3, 1, 0, 1, 0, 0, "c", "c~", "c", "c~");
    ATOOLS::s_kftable[6] = new ATOOLS::Particle_Info(6, 1000.0, 0.0, 2, 3, 1, 0, 1, 0, 1, "t", "t~", "t", "t~");
    ATOOLS::s_kftable[1] = new ATOOLS::Particle_Info(1, 1000.0, 0.0, -1, 3, 1, 0, 1, 0, 0, "d", "d~", "d", "d~");
    ATOOLS::s_kftable[3] = new ATOOLS::Particle_Info(3, 1000.0, 0.0, -1, 3, 1, 0, 1, 0, 0, "s", "s~", "s", "s~");
    ATOOLS::s_kftable[5] = new ATOOLS::Particle_Info(5, 1000.0, 0.0, -1, 3, 1, 0, 1, 0, 1, "b", "b~", "b", "b~");
    ATOOLS::s_kftable[25] = new ATOOLS::Particle_Info(25, 1000.0, 0.0, 0, 0, 0, -1, 1, 0, 1, "H", "H", "H", "H");
    ATOOLS::s_kftable[250] = new ATOOLS::Particle_Info(250, 1000.0, 0.0, 0, 0, 0, -1, 1, 0, 1, "G0", "G0", "G0", "G0");
    ATOOLS::s_kftable[251] = new ATOOLS::Particle_Info(251, 1000.0, 0.0, 3, 0, 0, 0, 1, 0, 1, "G+", "G-", "G+", "G-");
    ATOOLS::s_kftable[1000014] = new ATOOLS::Particle_Info(1000014, 1000.0, 0.0, 0, 0, 1, 0, 1, 0, 1, "XD", "XD~", "XD", "XD~");
    ATOOLS::s_kftable[55] = new ATOOLS::Particle_Info(55, 1000.0, 0.0, 0, 0, 2, -1, 1, 0, 1, "Y1", "Y1", "Y1", "Y1");
    }
    void ParamInit()
    {
      DEBUG_FUNC(this);
      msg_Debugging() << std::setprecision(20);
      
    double cabi = p_dataread->GetEntry<double>("CKMBLOCK", 1);
    double gVYXD = p_dataread->GetEntry<double>("DMINPUTS", 1);
    double gAYXD = p_dataread->GetEntry<double>("DMINPUTS", 2);
    double gVYdd = p_dataread->GetEntry<double>("DMINPUTS", 3);
    double gAYdd = p_dataread->GetEntry<double>("DMINPUTS", 4);
    double gVYuu = p_dataread->GetEntry<double>("DMINPUTS", 5);
    double gAYuu = p_dataread->GetEntry<double>("DMINPUTS", 6);
    double gVYbb = p_dataread->GetEntry<double>("DMINPUTS", 7);
    double gAYbb = p_dataread->GetEntry<double>("DMINPUTS", 8);
    double gVYtt = p_dataread->GetEntry<double>("DMINPUTS", 9);
    double gAYtt = p_dataread->GetEntry<double>("DMINPUTS", 10);
    double aEWM1 = p_dataread->GetEntry<double>("SMINPUTS", 1);
    double Gf = p_dataread->GetEntry<double>("SMINPUTS", 2);
    double aS = p_dataread->GetEntry<double>("SMINPUTS", 3);
    double ymb = p_dataread->GetEntry<double>("YUKAWA", 5);
    double ymt = p_dataread->GetEntry<double>("YUKAWA", 6);
    double ymtau = p_dataread->GetEntry<double>("YUKAWA", 15);
    double MZ = p_dataread->GetEntry<double>("MASS", 23);
    double MTA = p_dataread->GetEntry<double>("MASS", 15);
    double MT = p_dataread->GetEntry<double>("MASS", 6);
    double MB = p_dataread->GetEntry<double>("MASS", 5);
    double MH = p_dataread->GetEntry<double>("MASS", 25);
    double MXD = p_dataread->GetEntry<double>("MASS", 1000014);
    double MY1 = p_dataread->GetEntry<double>("MASS", 55);
    double WZ = p_dataread->GetEntry<double>("DECAY", 23);
    double WW = p_dataread->GetEntry<double>("DECAY", 24);
    double WT = p_dataread->GetEntry<double>("DECAY", 6);
    double WH = p_dataread->GetEntry<double>("DECAY", 25);
    double WXD = p_dataread->GetEntry<double>("DECAY", 1000014);
    double WY1 = p_dataread->GetEntry<double>("DECAY", 55);
    double ZERO = ToDouble(0.0);
    DEBUG_VAR(ZERO);
    Complex CKM1x1 = cos(cabi);
    DEBUG_VAR(CKM1x1);
    Complex CKM1x2 = sin(cabi);
    DEBUG_VAR(CKM1x2);
    Complex CKM2x1 = (-sin(cabi));
    DEBUG_VAR(CKM2x1);
    Complex CKM2x2 = cos(cabi);
    DEBUG_VAR(CKM2x2);
    double aEW = ToDouble((1.0/aEWM1));
    DEBUG_VAR(aEW);
    double G = ToDouble(((2.0*sqrt(aS))*sqrt(M_PI)));
    DEBUG_VAR(G);
    double MW = ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0))))))));
    DEBUG_VAR(MW);
    double ee = ToDouble(((2.0*sqrt(aEW))*sqrt(M_PI)));
    DEBUG_VAR(ee);
    double sw2 = ToDouble((1.0-(pow(MW,2.0)/pow(MZ,2.0))));
    DEBUG_VAR(sw2);
    double cw = ToDouble(sqrt((1.0-sw2)));
    DEBUG_VAR(cw);
    double sw = ToDouble(sqrt(sw2));
    DEBUG_VAR(sw);
    double g1 = ToDouble((ee/cw));
    DEBUG_VAR(g1);
    double gw = ToDouble((ee/sw));
    DEBUG_VAR(gw);
    double vev = ToDouble((((2.0*MW)*sw)/ee));
    DEBUG_VAR(vev);
    double lam = ToDouble((pow(MH,2.0)/(2.0*pow(vev,2.0))));
    DEBUG_VAR(lam);
    double yb = ToDouble(((ymb*sqrt(2.0))/vev));
    DEBUG_VAR(yb);
    double yt = ToDouble(((ymt*sqrt(2.0))/vev));
    DEBUG_VAR(yt);
    double ytau = ToDouble(((ymtau*sqrt(2.0))/vev));
    DEBUG_VAR(ytau);
    double muH = ToDouble(sqrt((lam*pow(vev,2.0))));
    DEBUG_VAR(muH);
    Complex I1c33 = yb;
    DEBUG_VAR(I1c33);
    Complex I2c33 = yt;
    DEBUG_VAR(I2c33);
    Complex I3c33 = yt;
    DEBUG_VAR(I3c33);
    Complex I4c33 = yb;
    DEBUG_VAR(I4c33);
    ATOOLS::Flavour(22).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(22).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(22).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(23).SetWidth(ToDouble(WZ));
    ATOOLS::Flavour(23).SetMass(ToDouble(MZ));
    ATOOLS::Flavour(23).SetHadMass(ToDouble(MZ));
    ATOOLS::Flavour(24).SetWidth(ToDouble(WW));
    ATOOLS::Flavour(24).SetMass(ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0)))))))));
    ATOOLS::Flavour(24).SetHadMass(ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0)))))))));
    ATOOLS::Flavour(21).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(21).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(21).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(9000001).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(9000001).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(9000001).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(9000002).SetWidth(ToDouble(WZ));
    ATOOLS::Flavour(9000002).SetMass(ToDouble(MZ));
    ATOOLS::Flavour(9000002).SetHadMass(ToDouble(MZ));
    ATOOLS::Flavour(9000003).SetWidth(ToDouble(WW));
    ATOOLS::Flavour(9000003).SetMass(ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0)))))))));
    ATOOLS::Flavour(9000003).SetHadMass(ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0)))))))));
    ATOOLS::Flavour(9000004).SetWidth(ToDouble(WW));
    ATOOLS::Flavour(9000004).SetMass(ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0)))))))));
    ATOOLS::Flavour(9000004).SetHadMass(ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0)))))))));
    ATOOLS::Flavour(82).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(82).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(82).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(12).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(12).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(12).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(14).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(14).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(14).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(16).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(16).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(16).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(11).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(11).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(11).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(13).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(13).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(13).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(15).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(15).SetMass(ToDouble(MTA));
    ATOOLS::Flavour(15).SetHadMass(ToDouble(MTA));
    ATOOLS::Flavour(2).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(2).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(2).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(4).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(4).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(4).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(6).SetWidth(ToDouble(WT));
    ATOOLS::Flavour(6).SetMass(ToDouble(MT));
    ATOOLS::Flavour(6).SetHadMass(ToDouble(MT));
    ATOOLS::Flavour(1).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(1).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(1).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(3).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(3).SetMass(ToDouble(0.0));
    ATOOLS::Flavour(3).SetHadMass(ToDouble(0.0));
    ATOOLS::Flavour(5).SetWidth(ToDouble(0.0));
    ATOOLS::Flavour(5).SetMass(ToDouble(MB));
    ATOOLS::Flavour(5).SetHadMass(ToDouble(MB));
    ATOOLS::Flavour(25).SetWidth(ToDouble(WH));
    ATOOLS::Flavour(25).SetMass(ToDouble(MH));
    ATOOLS::Flavour(25).SetHadMass(ToDouble(MH));
    ATOOLS::Flavour(250).SetWidth(ToDouble(WZ));
    ATOOLS::Flavour(250).SetMass(ToDouble(MZ));
    ATOOLS::Flavour(250).SetHadMass(ToDouble(MZ));
    ATOOLS::Flavour(251).SetWidth(ToDouble(WW));
    ATOOLS::Flavour(251).SetMass(ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0)))))))));
    ATOOLS::Flavour(251).SetHadMass(ToDouble(sqrt(((pow(MZ,2.0)/2.0)+sqrt(((pow(MZ,4.0)/4.0)-(((aEW*M_PI)*pow(MZ,2.0))/(Gf*sqrt(2.0)))))))));
    ATOOLS::Flavour(1000014).SetWidth(ToDouble(WXD));
    ATOOLS::Flavour(1000014).SetMass(ToDouble(MXD));
    ATOOLS::Flavour(1000014).SetHadMass(ToDouble(MXD));
    ATOOLS::Flavour(55).SetWidth(ToDouble(WY1));
    ATOOLS::Flavour(55).SetMass(ToDouble(MY1));
    ATOOLS::Flavour(55).SetHadMass(ToDouble(MY1));
    p_complexconstants->insert(std::make_pair(std::string("GC_1"),((-(ee*complex(0.0,1.0)))/3.0)));
    DEBUG_VAR((*p_complexconstants)["GC_1"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_2"),(((2.0*ee)*complex(0.0,1.0))/3.0)));
    DEBUG_VAR((*p_complexconstants)["GC_2"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_3"),(-(ee*complex(0.0,1.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_3"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_4"),(ee*complex(0.0,1.0))));
    DEBUG_VAR((*p_complexconstants)["GC_4"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_5"),(pow(ee,2.0)*complex(0.0,1.0))));
    DEBUG_VAR((*p_complexconstants)["GC_5"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_6"),((2.0*pow(ee,2.0))*complex(0.0,1.0))));
    DEBUG_VAR((*p_complexconstants)["GC_6"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_7"),((-pow(ee,2.0))/(2.0*cw))));
    DEBUG_VAR((*p_complexconstants)["GC_7"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_8"),((pow(ee,2.0)*complex(0.0,1.0))/(2.0*cw))));
    DEBUG_VAR((*p_complexconstants)["GC_8"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_9"),(pow(ee,2.0)/(2.0*cw))));
    DEBUG_VAR((*p_complexconstants)["GC_9"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_10"),(-G)));
    DEBUG_VAR((*p_complexconstants)["GC_10"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_11"),(complex(0.0,1.0)*G)));
    DEBUG_VAR((*p_complexconstants)["GC_11"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_12"),(complex(0.0,1.0)*pow(G,2.0))));
    DEBUG_VAR((*p_complexconstants)["GC_12"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_13"),(complex(0.0,1.0)*gAYbb)));
    DEBUG_VAR((*p_complexconstants)["GC_13"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_14"),(complex(0.0,1.0)*gAYdd)));
    DEBUG_VAR((*p_complexconstants)["GC_14"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_15"),(complex(0.0,1.0)*gAYtt)));
    DEBUG_VAR((*p_complexconstants)["GC_15"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_16"),(complex(0.0,1.0)*gAYuu)));
    DEBUG_VAR((*p_complexconstants)["GC_16"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_17"),(complex(0.0,1.0)*gAYXD)));
    DEBUG_VAR((*p_complexconstants)["GC_17"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_18"),(complex(0.0,1.0)*gVYbb)));
    DEBUG_VAR((*p_complexconstants)["GC_18"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_19"),(complex(0.0,1.0)*gVYdd)));
    DEBUG_VAR((*p_complexconstants)["GC_19"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_20"),(complex(0.0,1.0)*gVYtt)));
    DEBUG_VAR((*p_complexconstants)["GC_20"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_21"),(complex(0.0,1.0)*gVYuu)));
    DEBUG_VAR((*p_complexconstants)["GC_21"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_22"),(complex(0.0,1.0)*gVYXD)));
    DEBUG_VAR((*p_complexconstants)["GC_22"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_23"),I1c33));
    DEBUG_VAR((*p_complexconstants)["GC_23"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_24"),(-I2c33)));
    DEBUG_VAR((*p_complexconstants)["GC_24"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_25"),I3c33));
    DEBUG_VAR((*p_complexconstants)["GC_25"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_26"),(-I4c33)));
    DEBUG_VAR((*p_complexconstants)["GC_26"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_27"),((-2.0*complex(0.0,1.0))*lam)));
    DEBUG_VAR((*p_complexconstants)["GC_27"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_28"),((-4.0*complex(0.0,1.0))*lam)));
    DEBUG_VAR((*p_complexconstants)["GC_28"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_29"),((-6.0*complex(0.0,1.0))*lam)));
    DEBUG_VAR((*p_complexconstants)["GC_29"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_30"),((pow(ee,2.0)*complex(0.0,1.0))/(2.0*pow(sw,2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_30"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_31"),(-((pow(ee,2.0)*complex(0.0,1.0))/pow(sw,2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_31"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_32"),(((pow(cw,2.0)*pow(ee,2.0))*complex(0.0,1.0))/pow(sw,2.0))));
    DEBUG_VAR((*p_complexconstants)["GC_32"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_33"),((-ee)/(2.0*sw))));
    DEBUG_VAR((*p_complexconstants)["GC_33"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_34"),((-(ee*complex(0.0,1.0)))/(2.0*sw))));
    DEBUG_VAR((*p_complexconstants)["GC_34"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_35"),((ee*complex(0.0,1.0))/(2.0*sw))));
    DEBUG_VAR((*p_complexconstants)["GC_35"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_36"),((ee*complex(0.0,1.0))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_36"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_37"),(((CKM1x1*ee)*complex(0.0,1.0))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_37"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_38"),(((CKM1x2*ee)*complex(0.0,1.0))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_38"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_39"),(((CKM2x1*ee)*complex(0.0,1.0))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_39"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_40"),(((CKM2x2*ee)*complex(0.0,1.0))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_40"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_41"),(-(((cw*ee)*complex(0.0,1.0))/sw))));
    DEBUG_VAR((*p_complexconstants)["GC_41"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_42"),(((cw*ee)*complex(0.0,1.0))/sw)));
    DEBUG_VAR((*p_complexconstants)["GC_42"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_43"),((-pow(ee,2.0))/(2.0*sw))));
    DEBUG_VAR((*p_complexconstants)["GC_43"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_44"),((-(pow(ee,2.0)*complex(0.0,1.0)))/(2.0*sw))));
    DEBUG_VAR((*p_complexconstants)["GC_44"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_45"),(pow(ee,2.0)/(2.0*sw))));
    DEBUG_VAR((*p_complexconstants)["GC_45"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_46"),((((-2.0*cw)*pow(ee,2.0))*complex(0.0,1.0))/sw)));
    DEBUG_VAR((*p_complexconstants)["GC_46"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_47"),(((ee*complex(0.0,1.0))*sw)/(3.0*cw))));
    DEBUG_VAR((*p_complexconstants)["GC_47"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_48"),((((-2.0*ee)*complex(0.0,1.0))*sw)/(3.0*cw))));
    DEBUG_VAR((*p_complexconstants)["GC_48"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_49"),(((ee*complex(0.0,1.0))*sw)/cw)));
    DEBUG_VAR((*p_complexconstants)["GC_49"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_50"),(((-(cw*ee))/(2.0*sw))-((ee*sw)/(2.0*cw)))));
    DEBUG_VAR((*p_complexconstants)["GC_50"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_51"),(((-((cw*ee)*complex(0.0,1.0)))/(2.0*sw))-(((ee*complex(0.0,1.0))*sw)/(6.0*cw)))));
    DEBUG_VAR((*p_complexconstants)["GC_51"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_52"),((((cw*ee)*complex(0.0,1.0))/(2.0*sw))-(((ee*complex(0.0,1.0))*sw)/(6.0*cw)))));
    DEBUG_VAR((*p_complexconstants)["GC_52"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_53"),(((-((cw*ee)*complex(0.0,1.0)))/(2.0*sw))+(((ee*complex(0.0,1.0))*sw)/(2.0*cw)))));
    DEBUG_VAR((*p_complexconstants)["GC_53"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_54"),((((cw*ee)*complex(0.0,1.0))/(2.0*sw))+(((ee*complex(0.0,1.0))*sw)/(2.0*cw)))));
    DEBUG_VAR((*p_complexconstants)["GC_54"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_55"),((((cw*pow(ee,2.0))*complex(0.0,1.0))/sw)-(((pow(ee,2.0)*complex(0.0,1.0))*sw)/cw))));
    DEBUG_VAR((*p_complexconstants)["GC_55"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_56"),(((-(pow(ee,2.0)*complex(0.0,1.0)))+(((pow(cw,2.0)*pow(ee,2.0))*complex(0.0,1.0))/(2.0*pow(sw,2.0))))+(((pow(ee,2.0)*complex(0.0,1.0))*pow(sw,2.0))/(2.0*pow(cw,2.0))))));
    DEBUG_VAR((*p_complexconstants)["GC_56"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_57"),(((pow(ee,2.0)*complex(0.0,1.0))+(((pow(cw,2.0)*pow(ee,2.0))*complex(0.0,1.0))/(2.0*pow(sw,2.0))))+(((pow(ee,2.0)*complex(0.0,1.0))*pow(sw,2.0))/(2.0*pow(cw,2.0))))));
    DEBUG_VAR((*p_complexconstants)["GC_57"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_58"),((-(pow(ee,2.0)*vev))/(2.0*cw))));
    DEBUG_VAR((*p_complexconstants)["GC_58"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_59"),((pow(ee,2.0)*vev)/(2.0*cw))));
    DEBUG_VAR((*p_complexconstants)["GC_59"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_60"),(((-2.0*complex(0.0,1.0))*lam)*vev)));
    DEBUG_VAR((*p_complexconstants)["GC_60"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_61"),(((-6.0*complex(0.0,1.0))*lam)*vev)));
    DEBUG_VAR((*p_complexconstants)["GC_61"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_62"),((-(pow(ee,2.0)*vev))/(4.0*pow(sw,2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_62"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_63"),((-((pow(ee,2.0)*complex(0.0,1.0))*vev))/(4.0*pow(sw,2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_63"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_64"),(((pow(ee,2.0)*complex(0.0,1.0))*vev)/(2.0*pow(sw,2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_64"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_65"),((pow(ee,2.0)*vev)/(4.0*pow(sw,2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_65"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_66"),((-(pow(ee,2.0)*vev))/(2.0*sw))));
    DEBUG_VAR((*p_complexconstants)["GC_66"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_67"),((pow(ee,2.0)*vev)/(2.0*sw))));
    DEBUG_VAR((*p_complexconstants)["GC_67"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_68"),(((-(pow(ee,2.0)*vev))/(4.0*cw))-(((cw*pow(ee,2.0))*vev)/(4.0*pow(sw,2.0))))));
    DEBUG_VAR((*p_complexconstants)["GC_68"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_69"),(((pow(ee,2.0)*vev)/(4.0*cw))-(((cw*pow(ee,2.0))*vev)/(4.0*pow(sw,2.0))))));
    DEBUG_VAR((*p_complexconstants)["GC_69"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_70"),(((-(pow(ee,2.0)*vev))/(4.0*cw))+(((cw*pow(ee,2.0))*vev)/(4.0*pow(sw,2.0))))));
    DEBUG_VAR((*p_complexconstants)["GC_70"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_71"),(((pow(ee,2.0)*vev)/(4.0*cw))+(((cw*pow(ee,2.0))*vev)/(4.0*pow(sw,2.0))))));
    DEBUG_VAR((*p_complexconstants)["GC_71"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_72"),((((-((pow(ee,2.0)*complex(0.0,1.0))*vev))/2.0)-((((pow(cw,2.0)*pow(ee,2.0))*complex(0.0,1.0))*vev)/(4.0*pow(sw,2.0))))-((((pow(ee,2.0)*complex(0.0,1.0))*pow(sw,2.0))*vev)/(4.0*pow(cw,2.0))))));
    DEBUG_VAR((*p_complexconstants)["GC_72"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_73"),((((pow(ee,2.0)*complex(0.0,1.0))*vev)+((((pow(cw,2.0)*pow(ee,2.0))*complex(0.0,1.0))*vev)/(2.0*pow(sw,2.0))))+((((pow(ee,2.0)*complex(0.0,1.0))*pow(sw,2.0))*vev)/(2.0*pow(cw,2.0))))));
    DEBUG_VAR((*p_complexconstants)["GC_73"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_74"),(-(yb/sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_74"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_75"),(-((complex(0.0,1.0)*yb)/sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_75"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_76"),(-((complex(0.0,1.0)*yt)/sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_76"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_77"),(yt/sqrt(2.0))));
    DEBUG_VAR((*p_complexconstants)["GC_77"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_78"),(-ytau)));
    DEBUG_VAR((*p_complexconstants)["GC_78"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_79"),ytau));
    DEBUG_VAR((*p_complexconstants)["GC_79"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_80"),(-(ytau/sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_80"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_81"),(-((complex(0.0,1.0)*ytau)/sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_81"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_82"),(((ee*complex(0.0,1.0))*complexconjugate(CKM1x1))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_82"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_83"),(((ee*complex(0.0,1.0))*complexconjugate(CKM1x2))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_83"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_84"),(((ee*complex(0.0,1.0))*complexconjugate(CKM2x1))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_84"]);
    p_complexconstants->insert(std::make_pair(std::string("GC_85"),(((ee*complex(0.0,1.0))*complexconjugate(CKM2x2))/(sw*sqrt(2.0)))));
    DEBUG_VAR((*p_complexconstants)["GC_85"]);
      msg_Debugging() << std::setprecision(6);
    }
    
    void vertices_0 () {
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_47",ComplexConstant(std::string("GC_47"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_51",ComplexConstant(std::string("GC_51"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_47",ComplexConstant(std::string("GC_47"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_51",ComplexConstant(std::string("GC_51"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_47",ComplexConstant(std::string("GC_47"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_51",ComplexConstant(std::string("GC_51"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_36",ComplexConstant(std::string("GC_36"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_85",ComplexConstant(std::string("GC_85"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_83",ComplexConstant(std::string("GC_83"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_84",ComplexConstant(std::string("GC_84"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_82",ComplexConstant(std::string("GC_82"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_11",ComplexConstant(std::string("GC_11"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_T(3,2,1));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 1;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_11",ComplexConstant(std::string("GC_11"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_T(3,2,1));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 1;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 0;
    }
    void vertices_1 () {
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_11",ComplexConstant(std::string("GC_11"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_T(3,2,1));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 1;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_1",ComplexConstant(std::string("GC_1"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_1",ComplexConstant(std::string("GC_1"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_1",ComplexConstant(std::string("GC_1"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_48",ComplexConstant(std::string("GC_48"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_52",ComplexConstant(std::string("GC_52"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_48",ComplexConstant(std::string("GC_48"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_52",ComplexConstant(std::string("GC_52"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_48",ComplexConstant(std::string("GC_48"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_52",ComplexConstant(std::string("GC_52"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_36",ComplexConstant(std::string("GC_36"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_40",ComplexConstant(std::string("GC_40"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_39",ComplexConstant(std::string("GC_39"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
    }
    void vertices_2 () {
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_38",ComplexConstant(std::string("GC_38"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_37",ComplexConstant(std::string("GC_37"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_11",ComplexConstant(std::string("GC_11"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_T(3,2,1));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 1;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_11",ComplexConstant(std::string("GC_11"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_T(3,2,1));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 1;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_11",ComplexConstant(std::string("GC_11"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_T(3,2,1));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 1;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_2",ComplexConstant(std::string("GC_2"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_2",ComplexConstant(std::string("GC_2"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_2",ComplexConstant(std::string("GC_2"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)15,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)15,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_49",ComplexConstant(std::string("GC_49"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_53",ComplexConstant(std::string("GC_53"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)13,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)13,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_49",ComplexConstant(std::string("GC_49"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_53",ComplexConstant(std::string("GC_53"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
    }
    void vertices_3 () {
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)11,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)11,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_49",ComplexConstant(std::string("GC_49"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_53",ComplexConstant(std::string("GC_53"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV12");
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)16,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)15,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_36",ComplexConstant(std::string("GC_36"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)14,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)13,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_36",ComplexConstant(std::string("GC_36"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)12,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)11,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_36",ComplexConstant(std::string("GC_36"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)15,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)15,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_3",ComplexConstant(std::string("GC_3"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)13,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)13,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_3",ComplexConstant(std::string("GC_3"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)11,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)11,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_3",ComplexConstant(std::string("GC_3"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)16,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)16,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_54",ComplexConstant(std::string("GC_54"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)14,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)14,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_54",ComplexConstant(std::string("GC_54"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)12,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)12,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_54",ComplexConstant(std::string("GC_54"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
    }
    void vertices_4 () {
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)15,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)16,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_36",ComplexConstant(std::string("GC_36"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)13,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)14,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_36",ComplexConstant(std::string("GC_36"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)11,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)12,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_36",ComplexConstant(std::string("GC_36"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_32",ComplexConstant(std::string("GC_32"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVVV12");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 2;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_73",ComplexConstant(std::string("GC_73"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVS3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_57",ComplexConstant(std::string("GC_57"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVSS3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 2;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_46",ComplexConstant(std::string("GC_46"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVVV15");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 2;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1000014,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1000014,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)55,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_22",ComplexConstant(std::string("GC_22"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_17",ComplexConstant(std::string("GC_17"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().Lorentz.push_back("FFV10");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 1;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)2,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)55,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_21",ComplexConstant(std::string("GC_21"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_16",ComplexConstant(std::string("GC_16"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().Lorentz.push_back("FFV10");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 1;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)55,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_20",ComplexConstant(std::string("GC_20"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_15",ComplexConstant(std::string("GC_15"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().Lorentz.push_back("FFV10");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 1;
    }
    void vertices_5 () {
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)3,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)55,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_19",ComplexConstant(std::string("GC_19"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_14",ComplexConstant(std::string("GC_14"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().Lorentz.push_back("FFV10");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 1;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)1,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)55,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_19",ComplexConstant(std::string("GC_19"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_14",ComplexConstant(std::string("GC_14"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().Lorentz.push_back("FFV10");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 1;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)4,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)55,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_21",ComplexConstant(std::string("GC_21"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_16",ComplexConstant(std::string("GC_16"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().Lorentz.push_back("FFV10");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 1;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)55,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_18",ComplexConstant(std::string("GC_18"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_13",ComplexConstant(std::string("GC_13"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFV9");
      m_v.back().Lorentz.push_back("FFV10");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 1;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_31",ComplexConstant(std::string("GC_31"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVVV12");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 2;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)23,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_42",ComplexConstant(std::string("GC_42"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVV3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_5",ComplexConstant(std::string("GC_5"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVVV12");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 2;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_64",ComplexConstant(std::string("GC_64"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVS3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_30",ComplexConstant(std::string("GC_30"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVSS3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 2;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)22,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)24,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_4",ComplexConstant(std::string("GC_4"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("VVV3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
    }
    void vertices_6 () {
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)6,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_76",ComplexConstant(std::string("GC_76"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFS12");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)15,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)15,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_81",ComplexConstant(std::string("GC_81"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("FFS12");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,1) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)5,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_75",ComplexConstant(std::string("GC_75"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_Identity(1,2));
      m_v.back().Lorentz.push_back("FFS12");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_12",ComplexConstant(std::string("GC_12"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_12",ComplexConstant(std::string("GC_12"))) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_12",ComplexConstant(std::string("GC_12"))) );
      m_v.back().Color.push_back((UFO::UFO_CF_f(-1,1,4)*UFO::UFO_CF_f(2,3,-1)));
      m_v.back().Color.push_back((UFO::UFO_CF_f(-1,1,3)*UFO::UFO_CF_f(2,4,-1)));
      m_v.back().Color.push_back((UFO::UFO_CF_f(-1,1,2)*UFO::UFO_CF_f(3,4,-1)));
      m_v.back().Lorentz.push_back("VVVV14");
      m_v.back().Lorentz.push_back("VVVV13");
      m_v.back().Lorentz.push_back("VVVV11");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 2;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)21,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_10",ComplexConstant(std::string("GC_10"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_f(1,2,3));
      m_v.back().Lorentz.push_back("VVV3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 1;
      m_v.back().order[1]    = 0;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_61",ComplexConstant(std::string("GC_61"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("SSS3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 1;
      m_v.back().order[2]    = 0;
      m_v.push_back(Single_Vertex());
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().AddParticle( ATOOLS::Flavour((kf_code)25,0) );
      m_v.back().cpl.push_back( ATOOLS::Kabbala("GC_29",ComplexConstant(std::string("GC_29"))) );
      m_v.back().Color.push_back(UFO::UFO_CF_1());
      m_v.back().Lorentz.push_back("SSSS3");
      m_v.back().order.resize(3);
      m_v.back().order[0]    = 0;
      m_v.back().order[1]    = 2;
      m_v.back().order[2]    = 0;
    }
    void InitVertices()
    {
      
    vertices_0();
    vertices_1();
    vertices_2();
    vertices_3();
    vertices_4();
    vertices_5();
    vertices_6();
    }
  };
  
}

using namespace MODEL;

DECLARE_GETTER(DMsimp_Y1_XD,"DMsimp_Y1_XD",Model_Base,Model_Arguments);

Model_Base *ATOOLS::Getter<Model_Base,Model_Arguments,DMsimp_Y1_XD>::operator()(const Model_Arguments &args) const 
{
  return new DMsimp_Y1_XD(args.m_path, args.m_file, args.m_elementary);
}

void ATOOLS::Getter<Model_Base,Model_Arguments,DMsimp_Y1_XD>::PrintInfo(std::ostream &str,const size_t width) const 
{
  str<<"Automatically generated model \"DMsimp_Y1_XD\" based on UFO output"<<std::endl;
}
