#include "METOOLS/Explicit/Lorentz_Calculator.H"
#include "METOOLS/Currents/C_Scalar.H"
#include "METOOLS/Currents/C_Spinor.H"
#include "METOOLS/Currents/C_Vector.H"
#include "METOOLS/Explicit/Vertex.H"
#include "MODEL/Main/Single_Vertex.H"
#include "ATOOLS/Org/Message.H"
#include "ATOOLS/Org/Exception.H"

typedef std::complex<double> complex;

namespace METOOLS {

  template <typename SType>
  class VVV3_Calculator: public Lorentz_Calculator {
  public:
    
    VVV3_Calculator(const Vertex_Key &key):
      Lorentz_Calculator(key) {}

    std::string Label() const { return "VVV3"; }

    CObject *Evaluate(const CObject_Vector &jj)
    {

// if outgoind UFO-index is 0
if (p_v->V()->id.back()==0){
const CVec4 <SType> & j1 = *(jj[0]->Get< CVec4 <SType> >());
const ATOOLS::Vec4D & p1 = p_v->J(0)->P();
const CVec4 <SType> & j2 = *(jj[1]->Get< CVec4 <SType> >());
const ATOOLS::Vec4D & p2 = p_v->J(1)->P();
ATOOLS::Vec4D p0 = -p1-p2;
CVec4<SType>* j0 = NULL;
j0 = CVec4<SType>::New();
(*j0)[0] = ((p0[0] - (p1[0]) - (p0[0]) + p2[0] + p1[0] - (p2[0])) * (j1[0]) + (p0[1] - (p2[1])) * (j1[1]) + (p0[2] - (p2[2])) * (j1[2]) + (p0[3] - (p2[3])) * (j1[3])) * (j2[0]) - (((p0[1] - (p1[1])) * (j1[0]) + (p1[0] - (p2[0])) * (j1[1])) * (j2[1])) - (((p0[2] - (p1[2])) * (j1[0]) + (p1[0] - (p2[0])) * (j1[2])) * (j2[2])) - (((p0[3] - (p1[3])) * (j1[0]) + (p1[0] - (p2[0])) * (j1[3])) * (j2[3]));
(*j0)[1] = ((p1[1] - (p2[1])) * (j1[0]) + (p0[0] - (p1[0])) * (j1[1])) * (j2[0]) - (((p0[0] - (p2[0])) * (j1[0]) + (p0[1] - (p1[1]) - (p0[1]) + p2[1] + p1[1] - (p2[1])) * (j1[1]) - ((p0[2] - (p2[2])) * (j1[2])) - ((p0[3] - (p2[3])) * (j1[3]))) * (j2[1])) - (((p0[2] - (p1[2])) * (j1[1]) + (p1[1] - (p2[1])) * (j1[2])) * (j2[2])) - (((p0[3] - (p1[3])) * (j1[1]) + (p1[1] - (p2[1])) * (j1[3])) * (j2[3]));
(*j0)[2] = ((p1[2] - (p2[2])) * (j1[0]) + (p0[0] - (p1[0])) * (j1[2])) * (j2[0]) - (((p1[2] - (p2[2])) * (j1[1]) + (p0[1] - (p1[1])) * (j1[2])) * (j2[1])) - (((p0[0] - (p2[0])) * (j1[0]) - ((p0[1] - (p2[1])) * (j1[1])) + (p0[2] - (p1[2]) - (p0[2]) + p2[2] + p1[2] - (p2[2])) * (j1[2]) - ((p0[3] - (p2[3])) * (j1[3]))) * (j2[2])) - (((p0[3] - (p1[3])) * (j1[2]) + (p1[2] - (p2[2])) * (j1[3])) * (j2[3]));
(*j0)[3] = ((p1[3] - (p2[3])) * (j1[0]) + (p0[0] - (p1[0])) * (j1[3])) * (j2[0]) - (((p1[3] - (p2[3])) * (j1[1]) + (p0[1] - (p1[1])) * (j1[3])) * (j2[1])) - (((p1[3] - (p2[3])) * (j1[2]) + (p0[2] - (p1[2])) * (j1[3])) * (j2[2])) - (((p0[0] - (p2[0])) * (j1[0]) - ((p0[1] - (p2[1])) * (j1[1])) - ((p0[2] - (p2[2])) * (j1[2])) + (p0[3] - (p1[3]) - (p0[3]) + p2[3] + p1[3] - (p2[3])) * (j1[3])) * (j2[3]));
j0->SetS(j1.S()|j2.S());
return j0;

}

// if outgoind UFO-index is 1
if (p_v->V()->id.back()==1){
const CVec4 <SType> & j0 = *(jj[1]->Get< CVec4 <SType> >());
const ATOOLS::Vec4D & p0 = p_v->J(1)->P();
const CVec4 <SType> & j2 = *(jj[0]->Get< CVec4 <SType> >());
const ATOOLS::Vec4D & p2 = p_v->J(0)->P();
ATOOLS::Vec4D p1 = -p0-p2;
CVec4<SType>* j1 = NULL;
j1 = CVec4<SType>::New();
(*j1)[0] = ((p0[0] - (p1[0]) - (p0[0]) + p2[0] + p1[0] - (p2[0])) * (j0[0]) - ((p1[1] - (p2[1])) * (j0[1])) - ((p1[2] - (p2[2])) * (j0[2])) - ((p1[3] - (p2[3])) * (j0[3]))) * (j2[0]) - (((p0[1] - (p1[1])) * (j0[0]) - ((p0[0] - (p2[0])) * (j0[1]))) * (j2[1])) - (((p0[2] - (p1[2])) * (j0[0]) - ((p0[0] - (p2[0])) * (j0[2]))) * (j2[2])) - (((p0[3] - (p1[3])) * (j0[0]) - ((p0[0] - (p2[0])) * (j0[3]))) * (j2[3]));
(*j1)[1] = -(((p0[1] - (p2[1])) * (j0[0]) - ((p0[0] - (p1[0])) * (j0[1]))) * (j2[0]) - (((p1[0] - (p2[0])) * (j0[0]) - ((p0[1] - (p1[1]) - (p0[1]) + p2[1] + p1[1] - (p2[1])) * (j0[1])) - ((p1[2] - (p2[2])) * (j0[2])) - ((p1[3] - (p2[3])) * (j0[3]))) * (j2[1])) + ((p0[2] - (p1[2])) * (j0[1]) - ((p0[1] - (p2[1])) * (j0[2]))) * (j2[2]) + ((p0[3] - (p1[3])) * (j0[1]) - ((p0[1] - (p2[1])) * (j0[3]))) * (j2[3]));
(*j1)[2] = -(((p0[2] - (p2[2])) * (j0[0]) - ((p0[0] - (p1[0])) * (j0[2]))) * (j2[0]) - (((p0[2] - (p2[2])) * (j0[1]) - ((p0[1] - (p1[1])) * (j0[2]))) * (j2[1])) - (((p1[0] - (p2[0])) * (j0[0]) - ((p1[1] - (p2[1])) * (j0[1])) - ((p0[2] - (p1[2]) - (p0[2]) + p2[2] + p1[2] - (p2[2])) * (j0[2])) - ((p1[3] - (p2[3])) * (j0[3]))) * (j2[2])) + ((p0[3] - (p1[3])) * (j0[2]) - ((p0[2] - (p2[2])) * (j0[3]))) * (j2[3]));
(*j1)[3] = -(((p0[3] - (p2[3])) * (j0[0]) - ((p0[0] - (p1[0])) * (j0[3]))) * (j2[0]) - (((p0[3] - (p2[3])) * (j0[1]) - ((p0[1] - (p1[1])) * (j0[3]))) * (j2[1])) - (((p0[3] - (p2[3])) * (j0[2]) - ((p0[2] - (p1[2])) * (j0[3]))) * (j2[2])) - (((p1[0] - (p2[0])) * (j0[0]) - ((p1[1] - (p2[1])) * (j0[1])) - ((p1[2] - (p2[2])) * (j0[2])) - ((p0[3] - (p1[3]) - (p0[3]) + p2[3] + p1[3] - (p2[3])) * (j0[3]))) * (j2[3])));
j1->SetS(j0.S()|j2.S());
return j1;

}

// if outgoind UFO-index is 2
if (p_v->V()->id.back()==2){
const CVec4 <SType> & j0 = *(jj[0]->Get< CVec4 <SType> >());
const ATOOLS::Vec4D & p0 = p_v->J(0)->P();
const CVec4 <SType> & j1 = *(jj[1]->Get< CVec4 <SType> >());
const ATOOLS::Vec4D & p1 = p_v->J(1)->P();
ATOOLS::Vec4D p2 = -p0-p1;
CVec4<SType>* j2 = NULL;
j2 = CVec4<SType>::New();
(*j2)[0] = ((p0[0] - (p1[0]) - (p0[0]) + p2[0] + p1[0] - (p2[0])) * (j0[0]) - ((p1[1] - (p2[1])) * (j0[1])) - ((p1[2] - (p2[2])) * (j0[2])) - ((p1[3] - (p2[3])) * (j0[3]))) * (j1[0]) + ((p0[1] - (p2[1])) * (j0[0]) - ((p0[0] - (p1[0])) * (j0[1]))) * (j1[1]) + ((p0[2] - (p2[2])) * (j0[0]) - ((p0[0] - (p1[0])) * (j0[2]))) * (j1[2]) + ((p0[3] - (p2[3])) * (j0[0]) - ((p0[0] - (p1[0])) * (j0[3]))) * (j1[3]);
(*j2)[1] = ((p0[1] - (p1[1])) * (j0[0]) - ((p0[0] - (p2[0])) * (j0[1]))) * (j1[0]) + ((p1[0] - (p2[0])) * (j0[0]) - ((p0[1] - (p1[1]) - (p0[1]) + p2[1] + p1[1] - (p2[1])) * (j0[1])) - ((p1[2] - (p2[2])) * (j0[2])) - ((p1[3] - (p2[3])) * (j0[3]))) * (j1[1]) + ((p0[2] - (p2[2])) * (j0[1]) - ((p0[1] - (p1[1])) * (j0[2]))) * (j1[2]) + ((p0[3] - (p2[3])) * (j0[1]) - ((p0[1] - (p1[1])) * (j0[3]))) * (j1[3]);
(*j2)[2] = ((p0[2] - (p1[2])) * (j0[0]) - ((p0[0] - (p2[0])) * (j0[2]))) * (j1[0]) - (((p0[2] - (p1[2])) * (j0[1]) - ((p0[1] - (p2[1])) * (j0[2]))) * (j1[1])) + ((p1[0] - (p2[0])) * (j0[0]) - ((p1[1] - (p2[1])) * (j0[1])) - ((p0[2] - (p1[2]) - (p0[2]) + p2[2] + p1[2] - (p2[2])) * (j0[2])) - ((p1[3] - (p2[3])) * (j0[3]))) * (j1[2]) + ((p0[3] - (p2[3])) * (j0[2]) - ((p0[2] - (p1[2])) * (j0[3]))) * (j1[3]);
(*j2)[3] = ((p0[3] - (p1[3])) * (j0[0]) - ((p0[0] - (p2[0])) * (j0[3]))) * (j1[0]) - (((p0[3] - (p1[3])) * (j0[1]) - ((p0[1] - (p2[1])) * (j0[3]))) * (j1[1])) - (((p0[3] - (p1[3])) * (j0[2]) - ((p0[2] - (p2[2])) * (j0[3]))) * (j1[2])) + ((p1[0] - (p2[0])) * (j0[0]) - ((p1[1] - (p2[1])) * (j0[1])) - ((p1[2] - (p2[2])) * (j0[2])) - ((p0[3] - (p1[3]) - (p0[3]) + p2[3] + p1[3] - (p2[3])) * (j0[3]))) * (j1[3]);
j2->SetS(j0.S()|j1.S());
return j2;

}

    }

  };// end of class VVV3_Calculator

  template class VVV3_Calculator<double>;

}// end of namespace METOOLS

using namespace METOOLS;

DECLARE_GETTER(VVV3_Calculator<double>,"DVVV3",
	       Lorentz_Calculator,Vertex_Key);
Lorentz_Calculator *ATOOLS::Getter
<Lorentz_Calculator,Vertex_Key,VVV3_Calculator<double> >::
operator()(const Vertex_Key &key) const
{ return new VVV3_Calculator<double>(key); }

void ATOOLS::Getter<Lorentz_Calculator,Vertex_Key,
		    VVV3_Calculator<double> >::
PrintInfo(std::ostream &str,const size_t width) const
{ str<<"VVV3 vertex"; }
