#include "METOOLS/Explicit/Lorentz_Calculator.H"
#include "METOOLS/Currents/C_Scalar.H"
#include "METOOLS/Currents/C_Spinor.H"
#include "METOOLS/Currents/C_Vector.H"
#include "METOOLS/Explicit/Vertex.H"
#include "MODEL/Main/Single_Vertex.H"
#include "ATOOLS/Org/Message.H"
#include "ATOOLS/Org/Exception.H"

typedef std::complex<double> complex;

namespace METOOLS {

  template <typename SType>
  class VVVV12_Calculator: public Lorentz_Calculator {
  public:
    
    VVVV12_Calculator(const Vertex_Key &key):
      Lorentz_Calculator(key) {}

    std::string Label() const { return "VVVV12"; }

    CObject *Evaluate(const CObject_Vector &jj)
    {

// if outgoind UFO-index is 0
if (p_v->V()->id.back()==0){
const CVec4 <SType> & j1 = *(jj[0]->Get< CVec4 <SType> >());
const CVec4 <SType> & j2 = *(jj[1]->Get< CVec4 <SType> >());
const CVec4 <SType> & j3 = *(jj[2]->Get< CVec4 <SType> >());
CVec4<SType>* j0 = NULL;
j0 = CVec4<SType>::New();
(*j0)[0] = -(((j1[1]) * (j2[1]) + (j1[2]) * (j2[2]) + (j1[3]) * (j2[3])) * (j3[0]) + ((j1[1]) * (j2[0]) + (-2.0) * ((j1[0]) * (j2[1]))) * (j3[1]) + ((j1[2]) * (j2[0]) + (-2.0) * ((j1[0]) * (j2[2]))) * (j3[2]) + ((j1[3]) * (j2[0]) + (-2.0) * ((j1[0]) * (j2[3]))) * (j3[3]));
(*j0)[1] = ((-2.0) * ((j1[1]) * (j2[0])) + (j1[0]) * (j2[1])) * (j3[0]) + ((j1[0]) * (j2[0]) - ((j1[2]) * (j2[2])) - ((j1[3]) * (j2[3]))) * (j3[1]) - (((j1[2]) * (j2[1]) + (-2.0) * ((j1[1]) * (j2[2]))) * (j3[2])) - (((j1[3]) * (j2[1]) + (-2.0) * ((j1[1]) * (j2[3]))) * (j3[3]));
(*j0)[2] = ((-2.0) * ((j1[2]) * (j2[0])) + (j1[0]) * (j2[2])) * (j3[0]) - (((-2.0) * ((j1[2]) * (j2[1])) + (j1[1]) * (j2[2])) * (j3[1])) + ((j1[0]) * (j2[0]) - ((j1[1]) * (j2[1])) - ((j1[3]) * (j2[3]))) * (j3[2]) - (((j1[3]) * (j2[2]) + (-2.0) * ((j1[2]) * (j2[3]))) * (j3[3]));
(*j0)[3] = ((-2.0) * ((j1[3]) * (j2[0])) + (j1[0]) * (j2[3])) * (j3[0]) - (((-2.0) * ((j1[3]) * (j2[1])) + (j1[1]) * (j2[3])) * (j3[1])) - (((-2.0) * ((j1[3]) * (j2[2])) + (j1[2]) * (j2[3])) * (j3[2])) + ((j1[0]) * (j2[0]) - ((j1[1]) * (j2[1])) - ((j1[2]) * (j2[2]))) * (j3[3]);
j0->SetS(j1.S()|j2.S()|j3.S());
return j0;

}

// if outgoind UFO-index is 1
if (p_v->V()->id.back()==1){
const CVec4 <SType> & j0 = *(jj[2]->Get< CVec4 <SType> >());
const CVec4 <SType> & j2 = *(jj[0]->Get< CVec4 <SType> >());
const CVec4 <SType> & j3 = *(jj[1]->Get< CVec4 <SType> >());
CVec4<SType>* j1 = NULL;
j1 = CVec4<SType>::New();
(*j1)[0] = -(((j0[1]) * (j2[1]) + (j0[2]) * (j2[2]) + (j0[3]) * (j2[3])) * (j3[0]) + ((j0[1]) * (j2[0]) + (-2.0) * ((j0[0]) * (j2[1]))) * (j3[1]) + ((j0[2]) * (j2[0]) + (-2.0) * ((j0[0]) * (j2[2]))) * (j3[2]) + ((j0[3]) * (j2[0]) + (-2.0) * ((j0[0]) * (j2[3]))) * (j3[3]));
(*j1)[1] = ((-2.0) * ((j0[1]) * (j2[0])) + (j0[0]) * (j2[1])) * (j3[0]) + ((j0[0]) * (j2[0]) - ((j0[2]) * (j2[2])) - ((j0[3]) * (j2[3]))) * (j3[1]) - (((j0[2]) * (j2[1]) + (-2.0) * ((j0[1]) * (j2[2]))) * (j3[2])) - (((j0[3]) * (j2[1]) + (-2.0) * ((j0[1]) * (j2[3]))) * (j3[3]));
(*j1)[2] = ((-2.0) * ((j0[2]) * (j2[0])) + (j0[0]) * (j2[2])) * (j3[0]) - (((-2.0) * ((j0[2]) * (j2[1])) + (j0[1]) * (j2[2])) * (j3[1])) + ((j0[0]) * (j2[0]) - ((j0[1]) * (j2[1])) - ((j0[3]) * (j2[3]))) * (j3[2]) - (((j0[3]) * (j2[2]) + (-2.0) * ((j0[2]) * (j2[3]))) * (j3[3]));
(*j1)[3] = ((-2.0) * ((j0[3]) * (j2[0])) + (j0[0]) * (j2[3])) * (j3[0]) - (((-2.0) * ((j0[3]) * (j2[1])) + (j0[1]) * (j2[3])) * (j3[1])) - (((-2.0) * ((j0[3]) * (j2[2])) + (j0[2]) * (j2[3])) * (j3[2])) + ((j0[0]) * (j2[0]) - ((j0[1]) * (j2[1])) - ((j0[2]) * (j2[2]))) * (j3[3]);
j1->SetS(j0.S()|j2.S()|j3.S());
return j1;

}

// if outgoind UFO-index is 2
if (p_v->V()->id.back()==2){
const CVec4 <SType> & j0 = *(jj[1]->Get< CVec4 <SType> >());
const CVec4 <SType> & j1 = *(jj[2]->Get< CVec4 <SType> >());
const CVec4 <SType> & j3 = *(jj[0]->Get< CVec4 <SType> >());
CVec4<SType>* j2 = NULL;
j2 = CVec4<SType>::New();
(*j2)[0] = (2.0) * (((j0[1]) * (j1[1]) + (j0[2]) * (j1[2]) + (j0[3]) * (j1[3])) * (j3[0])) + -(((j0[1]) * (j1[0]) + (j0[0]) * (j1[1])) * (j3[1])) - (((j0[2]) * (j1[0]) + (j0[0]) * (j1[2])) * (j3[2])) - (((j0[3]) * (j1[0]) + (j0[0]) * (j1[3])) * (j3[3]));
(*j2)[1] = ((j0[1]) * (j1[0]) + (j0[0]) * (j1[1])) * (j3[0]) + (-2.0) * (((j0[0]) * (j1[0]) - ((j0[2]) * (j1[2])) - ((j0[3]) * (j1[3]))) * (j3[1])) - (((j0[2]) * (j1[1]) + (j0[1]) * (j1[2])) * (j3[2])) - (((j0[3]) * (j1[1]) + (j0[1]) * (j1[3])) * (j3[3]));
(*j2)[2] = ((j0[2]) * (j1[0]) + (j0[0]) * (j1[2])) * (j3[0]) - (((j0[2]) * (j1[1]) + (j0[1]) * (j1[2])) * (j3[1])) + (-2.0) * (((j0[0]) * (j1[0]) - ((j0[1]) * (j1[1])) - ((j0[3]) * (j1[3]))) * (j3[2])) - (((j0[3]) * (j1[2]) + (j0[2]) * (j1[3])) * (j3[3]));
(*j2)[3] = ((j0[3]) * (j1[0]) + (j0[0]) * (j1[3])) * (j3[0]) - (((j0[3]) * (j1[1]) + (j0[1]) * (j1[3])) * (j3[1])) - (((j0[3]) * (j1[2]) + (j0[2]) * (j1[3])) * (j3[2])) + (-2.0) * (((j0[0]) * (j1[0]) - ((j0[1]) * (j1[1])) - ((j0[2]) * (j1[2]))) * (j3[3]));
j2->SetS(j0.S()|j1.S()|j3.S());
return j2;

}

// if outgoind UFO-index is 3
if (p_v->V()->id.back()==3){
const CVec4 <SType> & j0 = *(jj[0]->Get< CVec4 <SType> >());
const CVec4 <SType> & j1 = *(jj[1]->Get< CVec4 <SType> >());
const CVec4 <SType> & j2 = *(jj[2]->Get< CVec4 <SType> >());
CVec4<SType>* j3 = NULL;
j3 = CVec4<SType>::New();
(*j3)[0] = (2.0) * (((j0[1]) * (j1[1]) + (j0[2]) * (j1[2]) + (j0[3]) * (j1[3])) * (j2[0])) + -(((j0[1]) * (j1[0]) + (j0[0]) * (j1[1])) * (j2[1])) - (((j0[2]) * (j1[0]) + (j0[0]) * (j1[2])) * (j2[2])) - (((j0[3]) * (j1[0]) + (j0[0]) * (j1[3])) * (j2[3]));
(*j3)[1] = ((j0[1]) * (j1[0]) + (j0[0]) * (j1[1])) * (j2[0]) + (-2.0) * (((j0[0]) * (j1[0]) - ((j0[2]) * (j1[2])) - ((j0[3]) * (j1[3]))) * (j2[1])) - (((j0[2]) * (j1[1]) + (j0[1]) * (j1[2])) * (j2[2])) - (((j0[3]) * (j1[1]) + (j0[1]) * (j1[3])) * (j2[3]));
(*j3)[2] = ((j0[2]) * (j1[0]) + (j0[0]) * (j1[2])) * (j2[0]) - (((j0[2]) * (j1[1]) + (j0[1]) * (j1[2])) * (j2[1])) + (-2.0) * (((j0[0]) * (j1[0]) - ((j0[1]) * (j1[1])) - ((j0[3]) * (j1[3]))) * (j2[2])) - (((j0[3]) * (j1[2]) + (j0[2]) * (j1[3])) * (j2[3]));
(*j3)[3] = ((j0[3]) * (j1[0]) + (j0[0]) * (j1[3])) * (j2[0]) - (((j0[3]) * (j1[1]) + (j0[1]) * (j1[3])) * (j2[1])) - (((j0[3]) * (j1[2]) + (j0[2]) * (j1[3])) * (j2[2])) + (-2.0) * (((j0[0]) * (j1[0]) - ((j0[1]) * (j1[1])) - ((j0[2]) * (j1[2]))) * (j2[3]));
j3->SetS(j0.S()|j1.S()|j2.S());
return j3;

}

    }

  };// end of class VVVV12_Calculator

  template class VVVV12_Calculator<double>;

}// end of namespace METOOLS

using namespace METOOLS;

DECLARE_GETTER(VVVV12_Calculator<double>,"DVVVV12",
	       Lorentz_Calculator,Vertex_Key);
Lorentz_Calculator *ATOOLS::Getter
<Lorentz_Calculator,Vertex_Key,VVVV12_Calculator<double> >::
operator()(const Vertex_Key &key) const
{ return new VVVV12_Calculator<double>(key); }

void ATOOLS::Getter<Lorentz_Calculator,Vertex_Key,
		    VVVV12_Calculator<double> >::
PrintInfo(std::ostream &str,const size_t width) const
{ str<<"VVVV12 vertex"; }
